unit NetDirect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, Buttons, ShellAPI, ImgList, UrlMon;

const
  WM_URLSelected = wm_User + 1;

type
// TNetDialog ------------------------------------------------------------------
  TNetDialog = class(TForm)
    Panel1: TPanel;
    tvDirect: TTreeView;
    btnUpdate: TSpeedButton;
    StatusBar: TStatusBar;
    ImageList1: TImageList;
    procedure FormShow(Sender: TObject);
    procedure btnUpdateClick(Sender: TObject);
    procedure tvDirectClick(Sender: TObject);
  private
    slURLs,
    slItems    : TStringList;
    function GetImageIndex(URL : String) : Integer;
    function GetURL(Source, Dest: string): Boolean;
    procedure DoUpdate;
  public
    CompHandle : HWnd;
    aURL       : String;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetInfFileName : String;
    function GetSelectedURL : String;
  end;

// TNetDirect ------------------------------------------------------------------

  TURLSelectedEvent = procedure (Sender : TObject;
                                 aURL : String) of object;

  TNetDirect = class(TComponent)
  private
    fHWnd          : HWnd;
    fURL           : String;
    fOnURLSelected : TURLSelectedEvent;
    NetDialog      : TNetDialog;
    function GetTitle : String;
    procedure SetTitle(S : String);
  protected
    procedure CompWndProc(var Message: TMessage);
    procedure WMURLSelected(var Message: TMessage); message WM_URLSelected;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property Title : String read GetTitle write SetTitle;
    property URL : String read fURL write fURL;

    property OnURLSelected: TURLSelectedEvent read fOnURLSelected write fOnURLSelected;
  end;

procedure Register;

implementation

{$R *.DFM}

// TNetDialog ------------------------------------------------------------------

const
  MaxItems    = 99;
  MaxSubItems = 99;
  sItem       = 'item';
  sSubItem    = 'subitem';

  conError    = 'Connection error, ';
  httpError   = 'Remote server "Not Found" error, ';
  puMsg       = 'using previous update';
  abMsg       = 'request aborted';

constructor TNetDialog.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  slURLs := TStringList.Create;
  slItems := TStringList.Create;
end;

destructor TNetDialog.Destroy;

begin
  slURLs.Free;
  slItems.Free;
  Inherited Destroy;
end;

procedure TNetDialog.FormShow(Sender: TObject);

begin
  StatusBar.SimpleText := '';
end;

function TNetDialog.GetURL(Source, Dest: string): Boolean;
begin
  try
    Result := (UrlDownloadToFile(nil, PChar(source), PChar(Dest), 0, nil) = 0);
  except
    Result := False;
  end;
end;

function TNetDialog.GetInfFileName : String;

var
  I : Integer;
  S : String;

begin
  I := Length(aURL);
  S := '';
  while (I > 1) And (aURL[I] <> '/') do
  begin
    S := aURL[I] + S;
    Dec(I)
  end;
  Result := S;
end;

function TNetDialog.GetSelectedURL : String;

var I : Integer;

begin
  Result := '';
  with tvDirect do
    if selected <> nil then
      if Selected.Level > 0 then
      begin
        I := Integer(Selected.Data);
        Result := slURLs[I];
      end;
end;

function TNetDialog.GetImageIndex(URL : String) : Integer;

var
  I : Integer;
  S : String;

begin
  S := '';
  Result := 1;
  I := Length(URL);
  while (I > 1) and (URL[I] <> '.') do
  begin
    S := URL[I] + S;
    Dec(I);
  end;
  if S = 'zip' then
    Result := 2
  else
    if S = 'exe' then
      Result := 3;
end;

procedure TNetDialog.DoUpdate;

var
  Error,
  OldData      : Boolean;
  I, J, X      : Integer;
  S1, S2, S3   : String;
  rItem, cItem : TTreeNode;

begin
  slItems.Clear;
  slURLs.Clear;
  tvDirect.Items.Clear;
  S1 := ExtractFilePath(Application.ExeName) + GetInfFileName;
  OldData := FileExists(S1);
  Screen.Cursor := crHourGlass;
  StatusBar.SimpleText := 'Connecting to: ' + aURL;
  Error := Not GetURL(aURL, S1);
  StatusBar.SimpleText := '';
  Screen.Cursor := crDefault;
  if Error then                    // Connection error
    if OldData then
      begin
        slItems.LoadFromFile(S1);
        StatusBar.SimpleText := conError + puMsg;
      end
    else
      begin
        StatusBar.SimpleText := conError + abMsg;
        Exit;
      end
  Else
    slItems.LoadFromFile(S1);
  S2 := slItems[0];
  if Pos('404', S2) > 0 Then       // Connect OK, HTTP 404 - object not found
    if OldData then
      begin
        slItems.LoadFromFile(S1);
        StatusBar.SimpleText := httpError + puMsg;
      end
    else
      begin
        StatusBar.SimpleText := httpError + abMsg;
        Exit;
      end;
  with tvDirect.Items do
  begin
    for I := 0 to MaxItems do
    begin
      S1 := slItems.Values[sItem + IntToStr(I)];
      if S1 <> '' then
      begin
        rItem := Add(Nil, S1);
        rItem.ImageIndex := 0;
        for J := 0 to MaxSubItems do
        begin
          S2 := slItems.Values[sSubItem + IntToStr(I) + '-' + IntToStr(J)];
          if S2 <> '' then
          begin
            X := Pos('|', S2);
            if X = 0 then
              Continue;
            S3 := Trim(Copy(S2, X + 1, 128));
            S2 := Trim(Copy(S2, 1, X - 1));
            cItem := AddChild(rItem, S2);
            cItem.ImageIndex := GetImageIndex(S3);
            cItem.Data := Pointer(slURLs.Add(S3));
          end;
        end;
      end;
    end;
  end;
end;

procedure TNetDialog.btnUpdateClick(Sender: TObject);

begin
  DoUpdate;
end;

procedure TNetDialog.tvDirectClick(Sender: TObject);

var S : String;

begin
  S := GetSelectedURL;
  if S <> '' then
  begin
    SendMessage(CompHandle, WM_URLSelected, 0, 0);
    ShellExecute(Handle, 'open', PChar(S), nil, nil, SW_SHOW);
    tvDirect.Selected.Selected := False;
  end;
end;

// TNetDirect ------------------------------------------------------------------

constructor TNetDirect.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  fHWnd := AllocateHWnd(CompWndProc);
  NetDialog := TNetDialog.Create(Self);
  NetDialog.Visible := False;
end;

destructor TNetDirect.Destroy;

begin
  DeallocateHWnd(fHWnd);
  NetDialog.Free;
  Inherited Destroy;
end;

procedure TNetDirect.CompWndProc(var Message: TMessage);

begin
  Dispatch(Message);
end;

procedure TNetDirect.WMURLSelected(var Message: TMessage);

begin
  if Assigned(fOnURLSelected) then
    fOnURLSelected(Self, NetDialog.GetSelectedURL);
end;

function TNetDirect.GetTitle : String;

begin
  Result := NetDialog.Caption;
end;

procedure TNetDirect.SetTitle(S : String);

begin
  NetDialog.Caption := S;
end;

procedure TNetDirect.Execute;

begin
  with NetDialog do
  begin
    CompHandle := fHWnd;
    aURL := fURL;
    Show;
  end;
end;


procedure Register;
begin
  RegisterComponents('LGM', [TNetDirect]);
end;

end.
